/*
 * ]project-open[ FS Sync
 * Copyright (c) 2018 ]project-open[
 * Authors:
 * - Gonzalo Prez de Olaguer Crdoba <gonzalo.perez@project-open.com>
 * - Frank Bergmann <frank.bergmann@project-open.com>
 *
 * This code is licensed under the GNU GPL version 2.0 or later
 *
 * This is the application window script.
 * It creates the application window.
 *
 * This script requires Sencha Ext/JS 4.2.1
 */

/* Load Electron inter process communication (IPC) */
const {ipcRenderer} = require('electron')


Ext.Loader.setPath('POSync', '.');
Ext.Loader.setConfig({disableCaching: false});
Ext.require([
    'POSync.model.File',
    'POSync.model.Project',
    'POSync.model.Operation',

    'POSync.store.Files',
    'POSync.store.Projects',
    'POSync.store.Operations',

    'POSync.view.LoginPanel',
    'POSync.view.ProjectsPanel',

    'POSync.controller.Gui',
    'POSync.controller.Connection'
]);

var node_version = process.versions.node;
var chrome_version = process.versions.chrome;
var electron_version = process.versions.electron;
var extjs_version = Ext.getVersion().version;


/*
 * Debugging function that sets margin, border, padding and
 * background color to help identify the layout of the views
 */

var bg_colors = ["#ff0000", "#00ff00", "#0000ff", "#ffff00", "#ff00ff", "#00ffff" ];



/************************************************************
 *
 * GUI
 *
 ************************************************************/

/*
 * A form field to select a local directory.
 */
Ext.define('Ext.ux.DirField', {
    extend: 'Ext.form.field.Trigger',
    alias: 'widget.dirfield',

    /* Clicking the button opens a select directory dialog provided by electron */
    onTriggerClick: function() {
        const {dialog} = require('electron').remote;
        var path = dialog.showOpenDialog({
            title: 'Select the top directory to synchronize',
            properties: ['openDirectory']
        });
        if (path)
            this.setValue (path);
    }
});

function set_defaults (level, opts) {
    /* set to true(false) to enable(disable) coloured layouts */
    if (false) {
        if (!opts)
            opts = {};
        opts.style = {
            margin: '2px',
            padding: '2px',
            border: '1px solid black',
            background: bg_colors[level % 6],
        };
    }
    return opts;
}


/* The application GUI */
Ext.define('POSync.view.Viewport', {
    extend: 'Ext.container.Viewport',
    itemId: 'panel',
    layout: 'fit',
    defaults: set_defaults(0),
    items: {
        xtype: 'panel',
        title: false, // ']project-open[ Sync 1.0.0',
        defaults: set_defaults(1),
        layout: 'fit',
        items: {
            xtype: 'tabpanel',
            id: 'tabpanel',
            defaults: set_defaults(2, {layout: 'fit', defaults: set_defaults(3)}),
            items: [
		{
                    title: 'Login',
            	    hidden: false,
		    itemId: 'login_tab',
		    items: [
			Ext.create('POSync.view.LoginPanel')
		    ]
                }, {
                    title: 'Sync',
            	    hidden: false,
		    itemId: 'sync_tab',
                    items: {
                	xtype: 'grid',
                	itemId: 'operations',
                	store: Ext.create('POSync.store.Operations', {storeId: 'operations2'}),
                	emptyText: 'No operations, or all operations already executed',

                        selType: 'cellmodel',
                        plugins: [
                            Ext.create('Ext.grid.plugin.CellEditing', {
                                clicksToEdit: 1
                            })
                        ],
                	columns: [
                	    // {text: 'Project', dataIndex: 'project_id', flex: 1},
                	    {text: 'Operation', dataIndex: 'op', width: 80, editor: {
                                xtype: 'combobox',
                                forceSelection: true,
                                allowBlank: false,
                                editable: false,
                                store: Ext.create('Ext.data.Store', {
                                    fields: ['action', 'pretty'],
                                    data: [
                                        // {action: "download", pretty: "Download from Remote Server"},
                                        // {action: "upload", pretty: "Upload to Remote Server"}
                                        {action: "download", pretty: "Download"},
                                        {action: "upload", pretty: "Upload"}
                                    ]
                                }),
                                displayField: 'pretty',
                                valueField: 'action',
                                flex: 1
                            }, renderer: function(value) {
                                switch (value) {
                                case "conflict": return "Conflict";
                                case "upload": return "Upload";
                                case "download": return "Download";
                                default: return value;
                                }
                            }},
                	    {text: 'Path', dataIndex: 'file_path', flex: 2},
                	    {text: 'Name', dataIndex: 'file_name', flex: 3},
                	    {text: 'Status', dataIndex: 'status', width: 50},
                	    {text: 'Msg', dataIndex: 'err_msg', flex: 4},
                	    {text: 'Comment', dataIndex: 'comment', flex: 5}
                	]
                    }
                }, {
                    title: 'Projects',
                    icon: 'images/tray.png',
                    itemId: 'projects_tab',
            	    hidden: false,
		    items: [
			Ext.create('POSync.view.ProjectsPanel')
		    ]
/*
                    items: {
                	xtype: 'grid',
                	itemId: 'projects',
                	store: 'Projects',
                	emptyText: 'No projects available',
                	columns: [
                	    {text: 'Id', dataIndex: 'project_id', align: 'right', flex: 1},
                	    {text: 'Number', dataIndex: 'project_nr', flex: 2},
                	    {text: 'Name', dataIndex: 'project_name', flex: 7}
                	]
                    }
*/
                },{
            	    title: 'Local Files',
            	    hidden: false,
            	    itemId: 'local_files',
            	    items: {
            	        xtype: 'treepanel',
            	        itemId: 'localdir',
            	        store: Ext.create('POSync.store.Files', {
			    storeId: 'localFiles',
			    proxy: Ext.create('Ext.data.proxy.Memory'),
			    root: {expanded: true,children: []}
			}),
			rootVisible: false,
            	        emptyText: 'No files available',
            	        columns: [
            	            {text: 'Name', xtype: 'treecolumn', dataIndex: 'name', flex: 6},
            		    {text: 'Type', dataIndex: 'type', flex: 3, 						hidden: false},
          		    {text: 'Mime Type',dataIndex: 'mime_type',flex: 3, 				hidden: false},
            	            {text: 'Length', dataIndex: 'content_length', align: 'right', flex: 2, hidden: false},
            	            {text: 'Date', dataIndex: 'modification_date', flex: 6, 		hidden: false},
            	            {text: 'Path', dataIndex: 'path', flex: 6, 						hidden: false},
            	            {text: 'Sha1', dataIndex: 'sha1', flex: 6, 						hidden: true},
            	            {text: 'Expandable', dataIndex: 'expandable', flex: 1, 			hidden: true},
			    {text: 'Expanded', dataIndex: 'expanded', flex: 1, 				hidden: true}
			]
            	    }
            	},{
                    title: 'Remote Files',
            	    itemId: 'remote_files',
            	    hidden: false,
                    items: {
                	xtype: 'treepanel',
                	itemId: 'files',
                	store: Ext.create('POSync.store.Files', {storeId: 'remoteFiles'}),
			rootVisible: false,
                    	emptyText: 'No files available',
                    	tbar: {
                    	    xtype: 'combobox',
                    	    fieldLabel: 'Select a project',
                    	    store: 'Projects',
                    	    queryMode: 'local',
                    	    displayField: 'full_name',
                    	    valueField: 'project_id',
                    	    editable: false,
                    	    forceSelection: true,
                    	    emptyText: 'No project selected',
                    	},
                    	columns: [
                    	    {text: 'Name', xtype: 'treecolumn', dataIndex: 'name', flex: 6},
                    	    {text: 'Type', dataIndex: 'type', flex: 3, 						hidden: false},
          		    {text: 'Mime Type',dataIndex: 'mime_type',flex: 3, 				hidden: false},
                    	    {text: 'Length', dataIndex: 'content_length', align: 'right', flex: 2, hidden: false},
                    	    {text: 'Date', dataIndex: 'modification_date', flex: 6, 		hidden: false},
                    	    {text: 'Path', dataIndex: 'path', flex: 6, 						hidden: false},
                    	    {text: 'Title', dataIndex: 'title', flex: 6,					hidden: true},
                    	    {text: 'Description', dataIndex: 'description', flex: 6, 		hidden: true},
                    	    {text: 'Length', dataIndex: 'content_length', align: 'right', flex: 2, hidden: true},
                    	    {text: 'Id', dataIndex: 'id', align: 'right', flex: 1, 			hidden: true},
                    	    {text: 'Version', dataIndex: 'version_id', align: 'right', flex: 1, hidden: true},
                    	    {text: 'Sha1', dataIndex: 'sha1', flex: 6, 						hidden: true}
          		]
                    }
            	},{
            	    title: 'About',
            	    bodyPadding: 10,
            	    /* FIXME: mailto: doesn't work. electron handles it as a local file */
            	    html: [
            	        ']project-open[ Sync',
            	        'version 1.0.0',
            	        'Copyright &copy; 2019 ]project-open[ Business Solutions, S.L.',
            	        'Authors:',
            	        '- <a href="mailto:gonzalo.perez@project-open.com">Gonzalo Pérez de Olaguer Córdoba</a>',
            	        '- <a href="mailto:frank.bergmann@project-open.com">Frank Bergmann</a>',
            	        'This code is licensed under the GNU GPL version 2.0 or later',
            	        '',
            	        '<table>' +
            	            '<tr><td>Node version</td><td>'     + node_version     + '</td></tr>' +
            	            '<tr><td>Chrome version</td><td>'   + chrome_version   + '</td></tr>' +
            	            '<tr><td>Electron version</td><td>' + electron_version + '</td></tr>' +
            	            '<tr><td>ExtJS version</td><td>'    + extjs_version    + '</td></tr>' +
            	        '</table>'
            	    ].join('<br/>')
            	}
            ]
        },
	dockedItems: [{
	    xtype: 'toolbar',
	    dock: 'bottom',
	    ui: 'footer',
	    defaults: {minWidth: 50},
	    items: [
        	{ xtype: 'button', id: 'button_login', text: 'Login', action: 'login', hidden: false},
        	{ xtype: 'button', id: 'button_sync', text: 'Sync', action: 'sync', hidden: false},
		{
		    xtype    : 'textfield',
		    name     : 'comment',
		    id       : 'textfield_commit',
		    width    : 250,
		    emptyText: 'enter commit comment',
		    hidden   : true
		},
		{ xtype: 'button', id: 'button_commit', text: 'Commit changes', action: 'commit', hidden: true},
	    ]
	}]
	/*
	  buttons: [
          {text: 'Login', action: 'login'},
          {text: 'Sync', action: 'sync'}
          ]
	*/
    }

    /* End of POSync.view.Viewport */
});

/************************************************************
 *
 * Application
 *
 ************************************************************/

var app = Ext.application({
    name: 'POSync',
    id: 'POSync',
    itemId: 'POSync',
    autoCreateViewport: true,
    models: ['Project', 'File', 'Operation'],
    stores: ['Projects', 'Files', 'Operations'],
    controllers: ['Gui', 'Connection'],

    // Setup the tabs programmatically
    launch: function() {

        // Only enable login tab + button.
	var guiController = this.getGuiController();
	guiController.tabsAndButtons('login', 'login', 'login');
    }
});

