/*
 * Data related to a file.
 *
 * Only the fields required by this application are specified here.
 */


Ext.define('POSync.model.File', {
    extend:     'Ext.data.TreeModel',
    idProperty: 'id',
    fields: [
        'id',					// The system wide unique ID for this object (=file_id)
		'file_id',				// ]po[ object_id of the file
        'live_version_id',		// current active version of the file
        'name',					// The name (with extension) of the file or directory
        'path',					// The absolute path relative to the project folder, excluding name
        'extension',			// file extension
        'type',					// 'file' or 'directory'
        'mime_type',            // MIME type, empty for folders
        'local_p',				// 't' for a local file (client), 'f' for remote (server) file
        'iconCls',              // Used by ExtJS for the icon, seems to work
		'level',
        'expanded',             // default state for tree, true or false (without quotes)
        'expandable',			// not sure
        'leaf',					// true if node has no children
        'versions',				// array of version objects
		'children',				// FIXME: To be deleted
		'type_id',
		'status_id',

/*
		// Version attributes
        'version_id',			// The version of the file (cr_revisions.revision_id), empty for folders
        'title',				// Title of the file or folder
		'version_type_id',
		'version_status_id',
        'description',			// Description of the file or folder
        'content_length',		// File length in bytes (empty string for folders)
        'creation_date',        // Date of first creation
        'modification_date',	// Date of last modification
        'sha1',					// sha-1 hash of the content (empty string for folders)
*/
    ],



    /**
     * Returns the path of the file, starting with root.
     */
    getPathArray: function() {
        var me = this;

        // Skip the root and the 1st level folder, which refers to the project itself.
        if ("root" == me.get('id')) return [];

        var parent = me.parentNode;
        if (!parent) return [];

		// handle "root" separately, because it may not be a file instance
        var parentPath = [];
        if ("root" !== parent.get('id')) {
	        parentPath = parent.getPathArray();
		}

		// For a file return just the path (=parentPath)
        var fileType = me.get('type');
        if ("folder" != fileType) return parentPath;

		// For a folder, add the folder name to the path
        parentPath.push(me.get('name'));
        return parentPath;
    }

});